; Test timers in ATmega8515 - clock: internal 1MHz
; I demo the fast-PWM mode. Both counters are free running, but we change the value of
; compare registers, in order to change the duty cycle of PWM outputs.
; No interrupts are used here
; 3 outputs are used. Pins 1,15,29

.NOLIST
.INCLUDE "m8515def.inc"
.LIST

.def	pwm	= r23	; counter to determine the duty cycle

; Code starts here
.CSEG
.ORG $0000

; Reset- and Interrupt-vectors
		rjmp    main    ; 1 Reset-vector

delay:	ldi		r17,8	; delay routine
		ser		r18
ldel:	dec		r18
		brne	ldel
		ser		r18
		dec		r17
		brne	ldel
		ret

; **************** End of routines ***********

; ******************** Main program starts here ********************************

main:	ldi		r16,LOW(RAMEND) ;Initiate Stackpointer.
		out		SPL,r16
		ldi		r16,HIGH(RAMEND)
		out		SPH,r16
; ----------------------------------------------------------------------------------
; Timers notes - We have here one 8-bit timer and one 16-bit timer
; We also have one comparator for the 1st timer (8-bits) and two 16-bit comparators
; for the other 16-bit timer.
; Whenever an overflow or a compare match occurs, the corresponding output pin changes state.
; So the PWM wave is produced.
; This is a very good method for 3-channel Digital to Analog conversion.
; The last two channels can be 16-bit resolution, if 16-bit compare registers are used.
; Here I just use 8-bit PWM for all channels.
; Just low-pass filter the outputs. Choose appropriate prescaler. I use clock/8 = 1MHz/8 = 128KHz
; Check the program by just connecting a LED (with a 470 Ohm serial resistor) at each output pin
; See that intensity of leds vary from zero to maximum and then to minimum again.
; ----------------------------------------------------------------------------------------------
; Timer TCR0 is 8-bits and outputs at OC0: pin1
; TCCR0 Timer/Counter Control register
; Bit7 = FOC0 = 0 in fast PWM mode
; Fast PWM: (bit6=WGM00, bit3=WGM01) = (1,1) = FAST pwm MODE
; clock select for TCCR0
; bits 2,1,0 = cs02,cs01,cs0 function
;                 0   0   0  no source (stoped)
;                 0   0   1  clock
;                 0   1   0  clock/8
;                 0   1   1  clock/64
;                 1   0   0  clock/256
;                 1   0   1  clock/1024
; (bit5,bit4)=(com01,com00)=(1,0) non-inverting mode oc0=0 on match and oc0=1 on bottom.
; so TCCR0 = 0110 1101 = $6a (prescaler: clock/8) - clock=1MHz
; Timers TCCR0 and TCCRA share the same prescaler, but they can have the same prescaler settings
		ldi		r16,$6a		;Clock/8, fast PWM mode. Inverting mode - 8-bits, fast PWM 
		out		TCCR0,r16
; ----------------------------------------------------------------------------------------------
; TCNT1 is a 16-bits timer and is comprized by TCNT1H, TCNT1L
; It outputs at OC1A(pin15) and at OC1B(pin29)
; It has the compare registers: OCR1A, OCR1B and the control registers: TCCR1A, TCCR1B
; TCCR1A register:
;	BIT:	7		6		5		4		3		2		1		0
;			COM1A1	COM1A0	COM1B1	COM1B0	FOC1A	FOC1B	WGM11	WGM10
;			1		1		1		1		0		0		0		1
; For fast PWM, 8 bits counting, Top=ff, outputs OC1A/OC1B are set on compare match
; OC1A/OC1B are cleared at top
		ldi		r16,$f1
		out		TCCR1A,r16
; ----------------------------------------------------------------------------------------------
; TCCR1B
;   Bit:	7		6		5		4		3		2		1		0
;			ICNC1	ICED1	-		WGM13	WGM12	CS12	CS11	CS10
; 			0		0		0		0		1		0		1		0
; TCCR1B(2,1,0)=clock select: (1=fck, 2=fck/8, 3=fvl/64, 4=fck/256, 5=fck/1024)
		ldi		r16,10
		out		TCCR1B,r16

		ldi		r16,$ff		; Now set Data Direction of ports
		out		ddrA,r16
		out		ddrB,r16
		out		ddrC,r16
		out		ddrD,r16
		out		ddrE,r16
		clr		r16
		out		portA,r16	; clear all outputs
		out		portB,r16
		out		portC,r16
		out		portD,r16
		out		portE,r16

		clr		pwm
		out		OCR1AH,pwm	; 16-bit registers must be written
		out		OCR1BH,pwm	; writting high byte 1st

; Leds vary from zero to maximum and repeat so..
tcm1:	inc		pwm
tcm2:	out		OCR0,pwm
		out		OCR1AL,pwm
		out		OCR1BL,pwm
		rcall	delay	
		rjmp	tcm1
